from datetime import datetime
import streamlit as st
import pandas as pd
import plotly.express as px
from sqlalchemy import create_engine
import psycopg2

st.markdown(
    """
    <style>
    .stApp {
        background-color: #e6f7e6; 
        padding: 10px;
    }
    .header {
        text-align: center; 
        padding: 10px; 
        font-family: Arial, sans-serif; 
        background: linear-gradient(to right, red, orange, yellow, green, blue, indigo, violet);
        color: white; 
        border-radius: 8px;
    }
    .header h1 {
        margin: 0; 
        color: white; 
        text-shadow: 1px 1px 2px black;
    }
    .header p {
        color: #f0f0f0;
    }
    </style>

    <div class="header">
        <h1>Franchise Order Forecaster</h1>
        <p>Predicting demand quantities for optimized franchise operations</p>
    </div>
    """,
    unsafe_allow_html=True
)

# Database connection function
def get_database_connection():
    try:
        connection = psycopg2.connect(
            database="postgres",
            user="postgres",
            password="Welcome@1",
            host="localhost",
            port="5432"
        )
        return connection
    except Exception as e:
        st.error(f"Database connection failed: {str(e)}")
        return None

# Function to fetch data from database
def fetch_sales_data(franchise_id, season, sku=None):
    conn = get_database_connection()
    if conn:
        try:
            query = """
                SELECT franchise_id, season, sku, prediction_quantity 
                FROM public.sales_prediction 
                WHERE franchise_id = %s AND season = %s
            """
            params = [franchise_id, season]

            if sku:
                query += " AND sku = %s"
                params.append(sku)

            df = pd.read_sql_query(query, conn, params=params)
            conn.close()
            return df
        except Exception as e:
            st.error(f"Error fetching data: {str(e)}")
            conn.close()
            return None
    return None

current_year = datetime.now().year
future_years = [current_year + i for i in range(10)]
seasons = ["FALL", "HOLIDAY", "SPRING", "SUMMER"]
season_options = [f"{season}-{year}" for year in future_years for season in seasons]

franchise_id = st.number_input("FRANCHISE ID", min_value=30101, value=30101)
season = st.selectbox("SEASON", season_options)
sku = st.text_input("SKU (optional)", "")

# Get Data button
if st.button("Get Data"):
    if not season:
        st.warning("Please enter a season")

    # Fetch data
    df = fetch_sales_data(franchise_id, season, sku if sku else None)

    if df is not None and not df.empty:
        # Display the raw data
        st.dataframe(df.style.set_properties(**{'background-color': '#f9f9f9', 'color': '#333333'}), use_container_width=True)

        # Display charts only if SKU is not entered
        if not sku:
            # Bar Chart
            fig_bar = px.bar(
                df,
                x='sku',
                y='prediction_quantity',
                title='Prediction Quantity by SKU',
                labels={'prediction_quantity': 'Prediction Quantity', 'sku': 'SKU'},
            )
            fig_bar.update_layout(
                plot_bgcolor='#e6f7e6',
                paper_bgcolor='#e6f7e6',
                font=dict(color='#333333')
            )
            st.plotly_chart(fig_bar)

            # Pie Chart (optional example)
            fig_pie = px.pie(
                df,
                names='sku',
                values='prediction_quantity',
                title='Distribution of Prediction Quantity by SKU'
            )
            fig_pie.update_layout(
                plot_bgcolor='#e6f7e6',
                paper_bgcolor='#e6f7e6',
                font=dict(color='#333333')
            )
            st.plotly_chart(fig_pie)

    else:
        st.error("No data found for the given parameters")
