from datetime import datetime
import streamlit as st
import pandas as pd
import plotly.express as px
from sqlalchemy import create_engine
import psycopg2

st.markdown(
    """
    <style>
    .stApp {
        background-color: #f5f5f5; /* Neutral light grey for overall background */
        padding: 10px;
    }
    .header {
        text-align: center; 
        padding: 15px; 
        font-family: Arial, sans-serif; 
        background: linear-gradient(to right, #4caf50, #8bc34a); /* Green gradient for positivity */
        color: white; 
        border-radius: 8px;
    }
    .header h1 {
        margin: 0; 
        color: white; 
        text-shadow: 1px 1px 2px black;
    }
    .header p {
        color: #f0f0f0;
    }
    .stNumberInput > div, .stSelectbox > div, .stTextInput > div {
        background-color: #ffffff; /* White for clean input areas */
        color: #333333; /* Dark grey text for contrast */
        border: 1px solid #cccccc; /* Subtle border */
        border-radius: 5px;
    }
    .stButton > button {
        background-color: #4caf50; /* Green for primary action */
        color: white; 
        border: none; 
        padding: 10px 20px; 
        border-radius: 8px; 
        cursor: pointer; 
        font-size: 16px;
    }
    .stButton > button:hover {
        background-color: #388e3c; /* Slightly darker green for hover */
    }
    .stDataframe {
        border: 1px solid #e0e0e0;
        border-radius: 5px;
    }
    </style>

    <div class="header">
        <h1>Franchise Order Forecaster</h1>
        <p>Predicting demand quantities for optimized franchise operations</p>
    </div>
    """,
    unsafe_allow_html=True
)

# Database connection function
def get_database_connection():
    try:
        connection = psycopg2.connect(
            database="postgres",
            user="postgres",
            password="Welcome@1",
            host="localhost",
            port="5432"
        )
        return connection
    except Exception as e:
        st.error(f"Database connection failed: {str(e)}")
        return None

# Function to fetch data from database
def fetch_sales_data(franchise_id, season, sku=None):
    conn = get_database_connection()
    if conn:
        try:
            query = """
                SELECT franchise_id, season, sku, prediction_quantity 
                FROM public.sales_prediction 
                WHERE franchise_id = %s AND season = %s
            """
            params = [franchise_id, season]

            if sku:
                query += " AND sku = %s"
                params.append(sku)

            df = pd.read_sql_query(query, conn, params=params)
            conn.close()
            return df
        except Exception as e:
            st.error(f"Error fetching data: {str(e)}")
            conn.close()
            return None
    return None

current_year = datetime.now().year-4
future_years = [current_year + i for i in range(10)]
seasons = ["FALL", "HOLIDAY", "SPRING", "SUMMER"]
season_options = [f"{season}-{year}" for year in future_years for season in seasons]

franchise_id = st.number_input("FRANCHISE ID", min_value=30101, value=30101)
season = st.selectbox("SEASON", season_options)
sku = st.text_input("SKU (optional)", "")

# Get Data button
if st.button("Get Data"):
    if not season:
        st.warning("Please enter a season")

    # Fetch data
    df = fetch_sales_data(franchise_id, season, sku if sku else None)

    if df is not None and not df.empty:
        # Function to style the dataframe as markdown with column header styles
        def style_table(df):
            # Start with styled header
            header_html = """
            <style>
            .styled-table th {
                background-color: lightblue;
                color: black;
                font-weight: bold;
                text-align: left;
                padding: 10px;
            }
            .styled-table td {
                text-align: left;
                padding: 8px;
            }
            table {
                border-collapse: collapse;
                width: 100%;
            }
            tbody tr:nth-child(even) {
                background-color: #f9f9f9;
            }
            </style>
            <table class="styled-table">
                <thead>
                    <tr>
            """
            # Add column headers
            for col in df.columns:
                header_html += f"<th>{col}</th>"
            header_html += "</tr></thead><tbody>"

            # Add table rows
            for _, row in df.iterrows():
                header_html += "<tr>"
                for cell in row:
                    header_html += f"<td>{cell}</td>"
                header_html += "</tr>"

            header_html += "</tbody></table>"
            return header_html


        # Render styled table
        styled_html = style_table(df)
        st.markdown(styled_html, unsafe_allow_html=True)

        # Check if `prediction_quantity` has valid values
        if df['prediction_quantity'].notnull().any():
            # Display charts only if SKU is not entered
            if not sku:
                # Bar Chart
                fig_bar = px.bar(
                    df,
                    x='sku',
                    y='prediction_quantity',
                    title='Prediction Quantity by SKU',
                    labels={'prediction_quantity': 'Prediction Quantity', 'sku': 'SKU'},
                )
                fig_bar.update_layout(
                    plot_bgcolor='#f5f5f5',  # Grey background for chart area
                    paper_bgcolor='#ffffff',  # White surrounding chart background
                    font=dict(color='#333333')  # Grey font for labels
                )
                st.plotly_chart(fig_bar)

                # Pie Chart (optional example)
                fig_pie = px.pie(
                    df,
                    names='sku',
                    values='prediction_quantity',
                    title='Distribution of Prediction Quantity by SKU'
                )
                fig_pie.update_layout(
                    plot_bgcolor='#f5f5f5',
                    paper_bgcolor='#ffffff',
                    font=dict(color='#333333')
                )
                st.plotly_chart(fig_pie)
        else:
            st.error("Prediction quantity is null. No charts to display.")
    else:
        st.error("No data found for the given parameters.")

