from prophet import Prophet
import pandas as pd
import os

def convert_season_to_date(season):
    """
    Converts a season-year string (e.g., 'SPRING-2023') into a date string (e.g., '2023-03-01').

    Args:
        season (str): Season and year (e.g., 'SPRING-2023').

    Returns:
        str: Date string in 'YYYY-MM-DD' format.
    """
    season_month_mapping = {
        'HOLIDAY': 12,  # December
        'SPRING': 3,   # March
        'SUMMER': 6,   # June
        'FALL': 9      # September
    }

    season, year = season.split('-')
    month = season_month_mapping.get(season.upper(), 1)  # Default to January if season not found
    return f'{year}-{month:02d}-01'

def convert_date_to_season(date):
    """
    Converts a date (e.g., '2023-03-01') into a season-year string (e.g., 'SPRING-2023').
    """
    season_month_mapping = {
        1: 'HOLIDAY',  # January to February (Winter)
        2: 'HOLIDAY',
        3: 'SPRING',  # March to May (Spring)
        4: 'SPRING',
        5: 'SPRING',
        6: 'SUMMER',  # June to August (Summer)
        7: 'SUMMER',
        8: 'SUMMER',
        9: 'FALL',    # September to November (Fall)
        10: 'FALL',
        11: 'FALL',
        12: 'HOLIDAY' # December (Holiday season)
    }

    year = date.year
    month = date.month

    # Map month to season
    season = next((name for num, name in season_month_mapping.items() if month == num), None)

    if not season:
        raise ValueError(f"No season mapping found for month: {month}")
    return f'{season}-{year}'
def prepare_and_forecast():
    """
    Prepares data and forecasts demand for each SKU using Prophet.

    Args:
        data_file (str): Path to the CSV file containing historical data.
        sku_id (int, optional): Specific SKU ID to retrieve forecast. Defaults to None.

    Returns:
        dict: A dictionary of SKU forecasts.
    """
    # Read data
    print("Current working directory:", os.getcwd())
    file_path = 'data/History.csv'

    if not os.path.exists(file_path):
        raise FileNotFoundError(f"The file {file_path} does not exist. Please provide the correct path.")
    df = pd.read_csv(file_path)
    df['DATE'] = df['SEASON'].apply(convert_season_to_date)
    df['DATE'] = pd.to_datetime(df['DATE'])

    sku_forecasts = {}

    for sku in df['SKU'].unique():
        # Filter data for the current SKU
        sku_data = df[df['SKU'] == sku]
        # print(sku_data)
        # Group by Date and sum the quantities ordered
        df_aggregated = sku_data.groupby('DATE').agg({'QUANTITY_ORDERED': 'sum'}).reset_index()

        # Prepare data for Prophet
        sku_data_prophet = df_aggregated.rename(columns={'DATE': 'ds', 'QUANTITY_ORDERED': 'y'})

        # Initialize and fit the Prophet model
        model = Prophet(yearly_seasonality=True)
        model.fit(sku_data_prophet)
        last_date = sku_data_prophet['ds'].max()
        last_year = last_date.year
        future_dates = []

        # Generate dates for the next 2 years on a quarterly basis
        quarters = [3, 6, 9, 12]  # Representing January, April, July, October

        for year_offset in range(1,5):  # Forecast for the next 4 years
            for month in quarters:  # Iterate through quarterly months
                future_dates.append(pd.to_datetime(f"{last_year + year_offset}-{month:02d}-01"))

        # Create a DataFrame with the future dates
        future_seasonal_df = pd.DataFrame({'ds': future_dates})
        # Make future predictions (4 quarters ahead)
        # future = model.make_future_dataframe(periods=5, freq='Q')
        forecast = model.predict(future_seasonal_df)
        combined_df = pd.concat([sku_data_prophet, forecast[['ds', 'yhat', 'yhat_lower', 'yhat_upper']]])
        sku_forecasts[sku] = combined_df
        final_data = []
        # Iterate over each SKU and its forecast
        for sku, forecast_df in sku_forecasts.items():
            # Add season to the forecast DataFrame
            forecast_df['season'] = forecast_df['ds'].apply(lambda date: convert_date_to_season(date))
            forecast_df = forecast_df.rename(columns={'yhat': 'prediction_quantity'})
            # Select only the required columns and add SKU information
            sku_data = forecast_df.tail(16)[['season', 'prediction_quantity']].copy()
            sku_data['sku'] = sku
            # sku_data = forecast_df[['season', 'yhat']].copy()
            # sku_data['sku'] = sku  # Add SKU column
            # Append the transformed data to final_data list
            final_data.extend(sku_data.to_dict(orient='records'))
        # Convert the final_data list into a DataFrame for better viewing/manipulation
        final_df = pd.DataFrame(final_data)

        # Display the resulting DataFrame
        #print(final_df)
        # Optionally, print or plot the forecast for the current SKU
        # print(f"Forecast for SKU: {sku}")
        # print(forecast[['ds', 'yhat']].tail(4))  # Print forecasted values for the next 4 periods

    # # Return a specific SKU forecast if provided
    # if sku_id is not None:
    #     return sku_forecasts.get(sku_id, None)

    return final_df
