from fastapi import FastAPI, HTTPException
from typing import List
from pydantic import BaseModel
import asyncpg

# Import the database functions
from database import fetch_sales_history, get_db_connection

app = FastAPI()


class SalesRecord(BaseModel):
    sku_id: int
    franchise_id: int
    season: str
    sku: int
    quantity_ordered: int


async def insert_sales_data(sales_data: List[SalesRecord]):
    conn = await get_db_connection()

    try:
        async with conn.transaction():
            # Prepare the SQL query to insert data into the table
            insert_query = """
                INSERT INTO public.sales_history (sku_id, franchise_id, season, sku, quantity_ordered)
                VALUES ($1, $2, $3, $4, $5)
            """

            # Loop through the sales records and execute the insertion query
            for record in sales_data:
                await conn.execute(
                    insert_query,
                    record.sku_id,
                    record.franchise_id,
                    record.season,
                    record.sku,
                    record.quantity_ordered
                )
    except Exception as e:
        raise HTTPException(status_code=500, detail=f"Error inserting data: {str(e)}")
    finally:
        await conn.close()


# Endpoint to fetch all sales history
@app.get("/sales-history", response_model=List[SalesRecord])
async def get_sales_history():
    try:
        query = """
            SELECT sku_id, franchise_id, season, sku, quantity_ordered
            FROM public.sales_history
        """
        params = []

        # Fetch data from the database
        sales_data = await fetch_sales_history(query, params)

        sales_records = [
            SalesRecord(
                sku_id=record['sku_id'],
                franchise_id=record['franchise_id'],
                season=record['season'],
                sku=record['sku'],  # Directly assign sku as an integer
                quantity_ordered=record['quantity_ordered']
            )
            for record in sales_data
        ]

        return sales_records

    except Exception as e:
        raise HTTPException(status_code=500, detail=str(e))


# Endpoint to insert sales data via a POST request
@app.post("/sales-history")
async def create_sales_history(sales_data: List[SalesRecord]):
    try:
        await insert_sales_data(sales_data)
        return {"message": "Sales data successfully inserted."}

    except Exception as e:
        raise HTTPException(status_code=500, detail=f"Error inserting data: {str(e)}")
