from fastapi import FastAPI, HTTPException
from pydantic import BaseModel
from sqlalchemy import create_engine, Column, Integer, String, Float, Date
from sqlalchemy.orm import declarative_base
# from sqlalchemy.ext.declarative import declarative_base
from sqlalchemy.orm import sessionmaker

import forcastgenerator

# Database URL (replace with your actual database credentials)
DATABASE_URL = "postgresql://postgres:Welcome%401@localhost:5432/postgres"

# Create the FastAPI app instance
app = FastAPI()

# Initialize the database connection
engine = create_engine(DATABASE_URL)
SessionLocal = sessionmaker(autocommit=False, autoflush=False, bind=engine)
Base = declarative_base()

# Define the database table model for sales_prediction
class SalesPrediction(Base):
    __tablename__ = "sales_prediction"
    id = Column(Integer, primary_key=True, index=True)
    franchise_id = Column(Integer, index=True)
    season = Column(String, index=True)
    sku = Column(Integer, index=True)
    prediction_quantity = Column(Float)

# Pydantic model for the request body
class SalesPredictionRequest(BaseModel):
    franchise_id: int
    season: str
    sku: int

class SalesPredictionResponse(BaseModel):
    franchise_id: int
    season: str
    sku: int
    prediction_quantity: float

# Load the trained model (Replace with your actual model path)
def load_model():
    prediction_model = forcastgenerator.prepare_and_forecast()
    return prediction_model

def update_db_from_list_and_dataframe(object_list, df, franchise_id):
    """
    Updates the database based on matches between object_list and df.

    :param db: SQLAlchemy session object
    :param object_list: List of objects to update
    :param df: Pandas DataFrame with `sku`, `season`, and new values
    """
    db = SessionLocal()
    for obj in object_list:
        # Find matching row in the DataFrame
        matching_row = df[(df['sku'] == obj.sku) & (df['season'] == obj.season)]

        # If a match is found, update the database
        if not matching_row.empty:
            # Extract the new value(s) to be updated
            new_value = matching_row.iloc[0]['prediction_quantity']  # Replace 'value_column' with actual column name

            # Update the record in the database
            db.query(SalesPrediction).filter(
                SalesPrediction.sku == obj.sku,
                SalesPrediction.season == obj.season,
                SalesPrediction.franchise_id == franchise_id
            ).update({'prediction_quantity': int(new_value)})  # Replace 'quantity' with the actual field to update

    # Commit the changes
    db.commit()
    # db.refresh(sales_prediction)
def predict_quantity(season: str, sku: int, df) -> float:

    # Filter the dataframe for the given SKU and season
    filtered_df = df[(df['sku'] == sku) & (df['season'] == season)]

    # Check if the filtered dataframe is empty
    if filtered_df.empty:
        return None  # Return None if no record is found
    else:
        return filtered_df['prediction_quantity'].iloc[0]

    # Initialize model (for example purposes, loading on app startup)
franchise_id: int = 30101
model = load_model()
db = SessionLocal()
sales_prediction_db = db.query(SalesPrediction).filter(
    #franchise_id we are hard coding for now
    SalesPrediction.franchise_id == franchise_id
).all()
#storing all the prediction values to db
update_db_from_list_and_dataframe(sales_prediction_db, model,franchise_id)
model

# Route to handle PATCH request to update prediction in DB
@app.patch("/predict/{franchise_id}/{season}/{sku}", response_model=SalesPredictionResponse)
def update_sales_prediction(
    franchise_id: int,
    season: str,
    sku: int,
    request: SalesPredictionRequest
):
    db = SessionLocal()
    try:
        # Query the existing prediction record
        sales_prediction = db.query(SalesPrediction).filter(
            SalesPrediction.franchise_id == franchise_id,
            SalesPrediction.season == season,
            SalesPrediction.sku == sku
        ).first()

        if sales_prediction:
            # Return updated response
            return SalesPredictionResponse(
                franchise_id=sales_prediction.franchise_id,
                season=sales_prediction.season,
                sku=sales_prediction.sku,
                prediction_quantity=sales_prediction.prediction_quantity
            )
        else:
            raise HTTPException(status_code=404, detail="Record not found")
    except Exception as e:
        db.rollback()
        raise HTTPException(status_code=400, detail=str(e))
    finally:
        db.close()

if __name__ == "__main__":
    import uvicorn
    uvicorn.run(app, host="127.0.0.1", port=8000)